"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.executeLintAnalysis = executeLintAnalysis;
exports.fixEslintIssue = fixEslintIssue;
var _eslint = require("eslint");
var _path = _interopRequireDefault(require("path"));
var _fileUtils = require("../../utils/fileUtils");
var _getFileUtils = require("./getFileUtils");
var _filterReports = require("./filterReports");
var _logger = require("../../utils/logger");
var _child_process = require("child_process");
var _requestUtils = require("../../utils/requestUtils");
var _getSonarProperties = require("../sonar-dependencies/getSonarProperties");
/**
 * @function executeLintAnalysis - start lint analysis
 * @param {Object} cliParams -  Object represent cli Argument 
 * @returns {void}
 */

async function executeLintAnalysis(cliParams) {
  _logger.Logger.cliLog("success", '\n Start linting ...');
  var {
    env,
    dir = false
  } = cliParams;
  const {
    ruleConfigurationPath,
    impactBased,
    branchDiffPath
  } = (0, _fileUtils.getLintConfiguration)();
  if (env == 'ci' && dir || env == 'dev' && !dir) {
    let errorMessage = env == 'ci' ? 'expected valid  configuration' : env == 'dir' ? 'expected --dir path in cli' : 'issue in configuration';
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `\n running in ${env} mode. ${errorMessage}`);
    process.exit(0);
  }
  const reportPath = (0, _getFileUtils.getLintReportPath)();
  var {
    JsFiles,
    branch_diff_json,
    CssFiles
  } = (0, _getFileUtils.getFiles)(env, dir, branchDiffPath);
  if (cliParams.cmdExecuted == "lint-ci" || cliParams.cmdExecuted == "dev-ci") {
    const {
      isExemptionApplied,
      excludedFiles
    } = await (0, _requestUtils.checkForExemptionNeed)((0, _getSonarProperties.getProjectNameForExemption)(cliParams));
    isExemptionApplied ? JsFiles = (0, _getFileUtils.filterExemptionFiles)(excludedFiles, JsFiles) : null;
    isExemptionApplied == false && _logger.Logger.log(_logger.Logger.FAILURE_TYPE, 'Exemption is not required !');
  }
  const filteredLintReport = await startLint(branch_diff_json, JsFiles, env, impactBased, ruleConfigurationPath);
  _logger.Logger.cliLog(_logger.Logger.SUCCESS_TYPE, `\n Linting analysis completed successfully...`);
  (0, _fileUtils.writeFileContents)(reportPath, JSON.stringify(filteredLintReport));
}

/**
 * @function startLint -  lint files based on configuration given
 * @param {JSON} branchDiff - Git Diff JSON  (This JSON get by git diff api)
 * @param {Array<String>} toLintFiles - impact files from git diff json
 * @param {string} linterFilesDirectory - directory path to lint
 * @param {string} impactBased  - param used to lint based on impact or not
 * @param {string} ruleConfigurationPath - configuration file path 
 * @returns {Promise<any|Object>}
 * 
 * @note -  while run via linterFilesDirectory impactBased param not used
 */

async function startLint(branchDiff = null, toLintFiles, env, impactBased, ruleConfigurationPath) {
  var eslint = createLint(ruleConfigurationPath);
  var rulesArrayReport = [];
  return await new Promise(async resolve => {
    if (env == 'dev' && toLintFiles.length) {
      rulesArrayReport = await eslint.lintFiles(toLintFiles);
    }
    if (env == 'ci' && branchDiff) {
      const filterJsonMap = (0, _filterReports.filterDiffJson)(branchDiff);
      for (let i = 0; i < toLintFiles.length; i++) {
        var singleFileReport = (await eslint.lintFiles(toLintFiles[i])).pop();
        if (impactBased === true) {
          const {
            filePath
          } = singleFileReport;
          singleFileReport = (0, _filterReports.filterLintReportBasedOnImpactLine)(singleFileReport, filterJsonMap, filePath);
        }
        rulesArrayReport.push(singleFileReport);
      }
    }
    resolve(rulesArrayReport);
  });
}

/**
 * @function createLint - create lint object 
 * @param {string} ruleConfigurationPath - configuration file path 
 * @returns {Object}
 */

function createLint(ruleConfigurationPath) {
  try {
    if (!(0, _fileUtils.checkIfFileExists)(ruleConfigurationPath)) {
      throw new Error('not Found');
    }
    return new _eslint.ESLint({
      overrideConfig: require((0, _getFileUtils.resolvePath)(ruleConfigurationPath)),
      cwd: _path.default.resolve(process.cwd()),
      useEslintrc: false,
      allowInlineConfig: false
    });
  } catch (err) {
    _logger.Logger.error(_logger.Logger.FAILURE_TYPE, `Error: ${err.message}`);
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `Either Lint configuration not found or rule plugin are not installed ..`);
    process.exit(0);
  }
}

/**
 *  @function fixEslintIssue -  help to fix lint issue
 *  @param {Array<String>} option - cli commads as array after parsing
 *  @returns {void}
 */

function fixEslintIssue(option) {
  // if(option.length == 1 && !checkIfFileExists(path.resolve(option))){
  //   Logger.log(Logger.FAILURE_TYPE,`check whether given file or directory exist or not`)
  //   process.exit(0)
  // }
  const prefix = 'npx';
  const command = ["eslint", "--fix", "--ext", ".js,.jsx", ...option];
  try {
    const lint_process = (0, _child_process.spawn)(prefix, command);
    lint_process.stdout.on('data', data => {
      _logger.Logger.log(_logger.Logger.INFO_TYPE, data.toString());
    });
    lint_process.stderr.on('error', data => {
      _logger.Logger.log(_logger.Logger.INFO_TYPE, data.toString());
    });
    lint_process.on('close', close => {});
  } catch (error) {
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
    process.exit(0);
  }
}