"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.checkForExemptionNeed = checkForExemptionNeed;
exports.createBranchDiff = createBranchDiff;
exports.deleteProject_SonarQube = deleteProject_SonarQube;
exports.getExemptionFileInfoPath = getExemptionFileInfoPath;
exports.getGitProjectId = getGitProjectId;
exports.requestAPI = requestAPI;
exports.requestUtilsCLI = requestUtilsCLI;
var _child_process = require("child_process");
var _fs = require("fs");
var _fileUtils = require("./fileUtils");
var _getSonarProperties = require("../core/sonar-dependencies/getSonarProperties");
var _hash = require("./hash");
var _logger = require("./logger");
var _http = _interopRequireDefault(require("http"));
var _path = _interopRequireDefault(require("path"));
var _querystring = _interopRequireDefault(require("querystring"));
/**
 * @function requestAPI - native request api 
 * @param {JSON} apiDetails 
 * @param {JSON} body 
 * @param {JSON} options 
 * @returns Promise<JSON>
 */

function requestAPI(apiDetails, body, options = {}) {
  return new Promise(resolve => {
    var request = _http.default.request(apiDetails, function (res) {
      var chunks = [];
      res.on("data", function (chunk) {
        chunks.push(chunk);
      });
      res.on("end", function (chunk) {
        var body = Buffer.concat(chunks);
        resolve(body.toString());
      });
      res.on("error", function (error) {
        _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
      });
    });
    request.write(body);
    request.end();
  });
}

/**
 *@function requestUtilsCLI - method execute curl cli request
 * @returns {Promise<Buffer>}
 */

function requestUtilsCLI(command) {
  return (0, _child_process.execSync)(command, {
    shell: true
  });
}

/**
 * @function createBranchDiff - make giff diff api via curl api
 * @returns {void}
 */

function createBranchDiff() {
  var command;
  var {
    token,
    gitEndPoint,
    branchDiffPath,
    compareBranch,
    projectId
  } = (0, _fileUtils.getLintConfiguration)();
  if (projectId == "project-id") {
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `\n Make sure that project id is properly configured`);
    projectId = getGitProjectId();
  }
  const endPoint = `${gitEndPoint}/api/v4/projects/${projectId.toString()}/repository/compare?from=${compareBranch}&to=${(0, _getSonarProperties.getBranchName)()}`;
  try {
    command = `curl --header "PRIVATE-TOKEN:${(0, _hash.decrypt)(token, 12)}" "${endPoint}"`;
    (0, _fs.writeFileSync)(branchDiffPath, requestUtilsCLI(command));
  } catch (err) {
    _logger.Logger.log(_logger.Logger.INFO_TYPE, `\n INFO : If you are using a VPN and encounter an SSL certification issue, ensure that the proxy is enabled for SSH and shell connections.`);
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `\n Make sure that you have access to this repository ${(0, _getSonarProperties.getRepoName)()}`);
  }
}

/**
 * @function getGitProjectId -  retrives project id of repository
 * @returns string
 */

function getGitProjectId() {
  var command;
  const {
    token,
    gitEndPoint
  } = (0, _fileUtils.getLintConfiguration)();
  const endPoint = `${gitEndPoint}/api/v4/projects?search=${(0, _getSonarProperties.getRepoName)()}`;
  try {
    command = (0, _child_process.execSync)(`curl --header "PRIVATE-TOKEN:${(0, _hash.decrypt)(token, 12)}" "${endPoint}"`);
    return JSON.parse(command.toString()).pop().id;
  } catch (err) {
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `make sure that you have access to this repository ${(0, _getSonarProperties.getRepoName)()}`);
    _logger.Logger.log(_logger.Logger.INFO_TYPE, err);
  }
}

/**
 * @function checkForExemptionNeed -  returns array of Paths after filteration of response of sonarQube issue api
 * @param {String} projectName  - project name of lint report
 * @returns Array<String>
 */

function checkForExemptionNeed(projectName) {
  return new Promise(async resolve => {
    _logger.Logger.log(_logger.Logger.INFO_TYPE, '\n Checking if an exemption has been raised for this branch');
    const {
      metricServerHost,
      meticHandler_api_token
    } = (0, _fileUtils.getLintConfiguration)();
    try {
      const exclusionEndpoint = `${metricServerHost.toString()}/api/settings/values?keys=`;
      const exclusionParams = ['sonar.exclusions', 'sonar.issue.ignore.allfile', 'sonar.inclusions', 'sonar.issue.ignore.block', 'sonar.test.exclusions', 'sonar.issue.ignore.multicriteria', 'sonar.test.inclusions', 'sonar.issue.enforce.multicriteria', 'sonar.coverage.exclusions', `sonar.cpd.exclusions&component=${projectName}`];
      const api = exclusionEndpoint.concat(exclusionParams.join(',').endsWith(",") ? exclusionParams.substring(0, url.length() - 1) : exclusionParams);
      const command = (0, _child_process.execSync)(`curl --location "${api}" --header 'Authorization: Basic ${(0, _hash.decrypt)(meticHandler_api_token, 12)}'`).toString();
      (0, _fs.writeFileSync)(getExemptionFileInfoPath(), JSON.stringify(command.toString()));
    } catch (error) {
      _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `Issue in applying exemption`);
      _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
    }
    resolve(filterSonarQubeJSONResponse());
  });
}

/**
 * @function getExemptionFileInfoPath - returns path of exemption details
 * @returns string
 */

function getExemptionFileInfoPath() {
  return _path.default.resolve(__dirname, "..", "..", "exemptionInfo.json");
}

/**
 * @function filterSonarQubeJSONResponse - convert json response to desired format
 * @returns Array<Object>
 */
async function filterSonarQubeJSONResponse() {
  try {
    var _exemptionDetails$set;
    var criteriaKey = "sonar.issue.ignore.multicriteria";
    const exemptionDetails = JSON.parse(require(getExemptionFileInfoPath()));
    var excludedFiles;
    let isExemptionNeed = false;
    (exemptionDetails === null || exemptionDetails === void 0 || (_exemptionDetails$set = exemptionDetails.settings) === null || _exemptionDetails$set === void 0 ? void 0 : _exemptionDetails$set.length) && exemptionDetails.settings.forEach(element => {
      if (element.key == criteriaKey) {
        isExemptionNeed = true;
        excludedFiles = element.fieldValues.map(ele => ele.resourceKey);
      }
    });
    return {
      isExemptionApplied: isExemptionNeed,
      excludedFiles: [...new Set(excludedFiles)]
    };
  } catch (error) {
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `Exemption not applied for this branch :(`);
    return {
      isExemptionApplied: false,
      excludedFiles: []
    };
  }
}

/**
 * @function deleteProject_SonarQube - this function delete project created in given sonarQube instance
 * @param {string} projectName - project name to delete  
 * @returns {Promiseany>}
 */
async function deleteProject_SonarQube(projectName) {
  try {
    const {
      metricServerHost
    } = (0, _fileUtils.getLintConfiguration)();
    const encryptedSonarToken = (0, _hash.decrypt)(meticHandler_api_token, 12);
    const {
      hostname
    } = new URL(metricServerHost);
    const requestBody = _querystring.default.stringify({
      project: projectName
    });
    const apiDetails = {
      method: "POST",
      hostname: hostname,
      path: "/api/projects/delete",
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded',
        Authorization: `Basic ${encryptedSonarToken}`
      },
      maxRedirects: 20
    };
    return await requestAPI(apiDetails, requestBody);
  } catch (error) {
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, error);
  }
}
async function updateSonarQubeProjectKey(projectName) {}